@@ -102,6 +102,17 @@ class GenericView:
     def get_created_datetime_text(self):
         return self.context.created_datetime.isoformat(' ') if hasattr(self.context, 'created_datetime') else None
 
+    def sanitize_doc_created_datetime(self, doc):
+        '''
+        created_datetime field in doc (as supplied by DataService) is a Python datetime object and is not serializable
+        to JSON. If field exists on doc, convert to string in ISO format
+        '''
+        assert doc
+        assert elita.util.type_check.is_dictlike(doc)
+        if 'created_datetime' in doc:
+            doc['created_datetime'] = doc['created_datetime'].isoformat(' ')
+        return doc
+
     def run_async(self, name, job_type, data, callable, args):
         jid = action.run_async(self.datasvc, name, job_type, data, callable, args)
         return {
@@ -197,9 +208,14 @@ class GenericView:
         if self.permissionless:
             self.permissions = "read;write"
             return
-        if self.allow_pw_auth and 'auth_token' not in self.req.params:  # view sub-class responsible for verifying pw
-            self.permissions = "read;write"
-            return
+        if self.allow_pw_auth:
+            if 'auth_token' not in self.req.params and 'Auth-Token' not in self.req.headers:
+                # view sub-class responsible for verifying pw
+                self.permissions = "read;write" if "password" in self.req.params else ""
+                return
+            elif "password" in self.req.params:  # both pw and auth token provided, fail the request
+                self.permissions = ""
+                return
         if 'auth_token' in self.req.params or 'Auth-Token' in self.req.headers:
             if 'auth_token' in self.req.params and 'Auth-Token' in self.req.headers:
                 #if both are specified on one request it's an error condition
@@ -346,7 +362,7 @@ class ApplicationView(GenericView):
             return err
         self.datasvc.appsvc.UpdateApplication(self.context.app_name, self.body)
         return {
-            'modified_application': self.datasvc.appsvc.GetApplication(self.context.app_name)
+            'modified_application': self.sanitize_doc_created_datetime(self.datasvc.appsvc.GetApplication(self.context.app_name))
         }
 
 class ActionContainerView(GenericView):
@@ -519,7 +535,7 @@ class BuildView(GenericView):
             return err
         self.datasvc.buildsvc.UpdateBuild(self.context.app_name, self.context.build_name, self.body)
         return {
-            'modifed_build': self.datasvc.buildsvc.GetBuild(self.context.app_name, self.context.build_name)
+            'modifed_build': self.sanitize_doc_created_datetime(self.datasvc.buildsvc.GetBuild(self.context.app_name, self.context.build_name))
         }
 
     def DELETE(self):
@@ -601,7 +617,7 @@ class ServerView(GenericView):
             return err
         self.datasvc.serversvc.UpdateServer(self.context.name, self.body)
         return {
-            'modified_server': self.datasvc.serversvc.GetServer(self.context.name)
+            'modified_server': self.sanitize_doc_created_datetime(self.datasvc.serversvc.GetServer(self.context.name))
         }
 
     def DELETE(self):
@@ -628,7 +644,8 @@ class DeploymentContainerView(GenericView):
     def GET(self):
         return {
             'application': self.context.parent,
-            'deployments': self.datasvc.deploysvc.GetDeployments(self.context.parent)
+            'deployments': self.datasvc.deploysvc.GetDeployments(self.context.parent, sort="desc",
+                                                                 with_creation_time=True)
         }
 
     @validate_parameters(required_params=['build_name'])
@@ -884,7 +901,7 @@ class KeyPairView(GenericView):
             return err
         self.datasvc.keysvc.UpdateKeyPair(self.context.name, self.body)
         return {
-            'modifed_keypair': self.datasvc.keysvc.GetKeyPair(self.context.name)
+            'modifed_keypair': self.sanitize_doc_created_datetime(self.datasvc.keysvc.GetKeyPair(self.context.name))
         }
 
     def DELETE(self):
@@ -957,7 +974,7 @@ class GitProviderView(GenericView):
             return err
         self.datasvc.gitsvc.UpdateGitProvider(self.context.name)
         return {
-            'modifed_gitprovider': self.datasvc.gitsvc.GetGitProvider(self.context.name)
+            'modifed_gitprovider': self.sanitize_doc_created_datetime(self.datasvc.gitsvc.GetGitProvider(self.context.name))
         }
 
     def DELETE(self):
@@ -1061,7 +1078,7 @@ class GitRepoView(GenericView):
             return err
         self.datasvc.gitsvc.UpdateGitRepo(self.context.application, self.context.name, self.body)
         return {
-            'modified_gitrepo': self.datasvc.gitsvc.GetGitRepo(self.context.application, self.context.name)
+            'modified_gitrepo': self.sanitize_doc_created_datetime(self.datasvc.gitsvc.GetGitRepo(self.context.application, self.context.name))
         }
 
     def DELETE(self):
@@ -1229,7 +1246,7 @@ class GitDeployView(GenericView):
         self.datasvc.gitsvc.UpdateGitDeploy(self.context.application, self.context.name, self.body)
         gd = self.datasvc.gitsvc.GetGitDeploy(self.context.application, self.context.name)
         return {
-            'modified_gitdeploy': gd,
+            'modified_gitdeploy': self.sanitize_doc_created_datetime(gd),
             'message': self.run_async("create_gitdeploy", "async", {'gitdeploy': gd['name']},
                              elita.deployment.gitservice.create_gitdeploy, {'gitdeploy': gd})
         }
@@ -1361,7 +1378,7 @@ class GroupView(GenericView):
             return err
         self.datasvc.groupsvc.UpdateGroup(self.context.application, self.context.name, self.body)
         return {
-            'modified_group': self.datasvc.groupsvc.GetGroup(self.context.application, self.context.name)
+            'modified_group': self.sanitize_doc_created_datetime(self.datasvc.groupsvc.GetGroup(self.context.application, self.context.name))
         }
 
     def DELETE(self):
@@ -1464,7 +1481,7 @@ class PackageMapView(GenericView):
             return err
         self.datasvc.pmsvc.UpdatePackageMap(self.context.application, self.context.name, self.body)
         return {
-            'modified_packagemap': self.datasvc.pmsvc.GetPackageMap(self.context.application, self.context.name)
+            'modified_packagemap': self.sanitize_doc_created_datetime(self.datasvc.pmsvc.GetPackageMap(self.context.application, self.context.name))
         }
 
     def DELETE(self):
@@ -1554,17 +1571,15 @@ class UserView(GenericView):
                                "attributes": self.context.attributes,
                                "auth_token": self.datasvc.usersvc.GetUserTokens(username)})
 
-    def GET(self):
-        # another ugly. we want to support both password auth and valid tokens, but only tokens from the same user or
-        #  with the '_global' permission.
+    def check_password(self):
         if 'password' in self.req.params:
             if not self.context.validate_password(self.req.params['password']):
-                return self.Error(403, "incorrect password")
+                return False, self.Error(403, "incorrect password")
         elif ('auth_token' not in self.req.params) and ('Auth-Token' not in self.req.headers):
-            return self.Error(403, "password or auth token required")
+            return False, self.Error(403, "password or auth token required")
         else:
             if 'auth_token' in self.req.params and len(self.req.params.getall('auth_token')) > 1:  # multiple auth tokens
-                return self.Error(403, "incorrect authentication")
+                return False, self.Error(403, "incorrect authentication")
             token = self.req.params.getall('auth_token')[0] if 'auth_token' in self.req.params else self.req.headers['Auth-Token']
             authsvc = auth.UserPermissions(self.datasvc.usersvc, token, datasvc=self.datasvc)
             allowed_apps = authsvc.get_allowed_apps()
@@ -1577,7 +1592,15 @@ class UserView(GenericView):
                 logging.debug("valid_token: {}".format(authsvc.valid_token))
                 logging.debug("usernames: {}, {}".format(authsvc.username, self.context.username))
                 logging.debug("allowed_apps: {}".format(authsvc.get_allowed_apps()))
-                return self.Error(403, "bad token")
+                return False, self.Error(403, "bad token")
+        return True, None
+
+    def GET(self):
+        # another ugly. we want to support both password auth and valid tokens, but only tokens from the same user or
+        #  with the '_global' permission.
+        ok, err = self.check_password()
+        if not ok:
+            return err
         name = self.context.username
         if len(self.datasvc.usersvc.GetUserTokens(name)) == 0:
             self.datasvc.usersvc.NewToken(name)
@@ -1585,12 +1608,15 @@ class UserView(GenericView):
 
     @validate_parameters(json_body={})
     def PATCH(self):
+        ok, err = self.check_password()
+        if not ok:
+            return err
         ok, err = self.check_patch_keys()
         if not ok:
             return err
         self.datasvc.usersvc.UpdateUser(self.context.username, self.body)
         return {
-            'modified_user': self.datasvc.usersvc.GetUser(self.context.username)
+            'modified_user': self.sanitize_doc_created_datetime(self.datasvc.usersvc.GetUser(self.context.username))
         }
 
 
@@ -1664,9 +1690,11 @@ class TokenView(GenericView):
         return self.status_ok({"token_deleted": {"username": user, "token": token}})
 
 
-#this is the default 'root' view that gets called for every request
 @view_config(name="", renderer='json')
 def Action(context, request):
+    '''
+    Default unnamed 'root' view that gets called for every request that doesn't 404 during traversal
+    '''
     logger.debug("REQUEST: url: {}".format(request.url))
     logger.debug("REQUEST: context: {}".format(context.__class__.__name__))
     logger.debug("REQUEST: method: {}".format(request.method))
