@@ -3,7 +3,9 @@ import mock
 import elita.deployment.deploy
 import elita.deployment.salt_control
 from elita.deployment import gitservice
-from elita.models import DataService, BuildDataService, GitDataService, JobDataService, GroupDataService, DeploymentDataService
+from elita.models import DataService, BuildDataService, GitDataService, JobDataService, GroupDataService, \
+    DeploymentDataService, ActionService
+from elita.actions.action import RegisterHooks
 #import logging
 
 
@@ -42,6 +44,8 @@ def setup_mock_datasvc():
     mock_datasvc.attach_mock(mock.Mock(spec=GitDataService), "gitsvc")
     mock_datasvc.attach_mock(mock.Mock(spec=JobDataService), "jobsvc")
     mock_datasvc.attach_mock(mock.Mock(spec=DeploymentDataService), "deploysvc")
+    mock_datasvc.attach_mock(mock.Mock(spec=ActionService), "actionsvc")
+    mock_datasvc.actionsvc.attach_mock(mock.Mock(spec=RegisterHooks), "hooks")
     mock_datasvc.job_id = "fake_job_id"
     mock_datasvc.gitsvc.GetGitDeploy = return_gitdeploy
     mock_datasvc.buildsvc.GetBuild = return_build
